from typing import Any, Protocol


class logger_type(Protocol):
    def __call__(self, *args: Any, **kwargs: Any) -> Any:
        ...

    def debug(self, text: str) -> None:
        ...

    def info(self, text: str) -> None:
        ...

    def warning(self, text: str) -> None:
        ...

    def error(self, text: str) -> None:
        ...

    def critical(self, text: str) -> None:
        ...

    def traceback(self, text: str) -> None:
        ...

    def contextualise(self, text: str) -> None:
        ...



import logging
from functools import wraps
from typing import Callable, ParamSpec, TypeVar

from prungo.models import logger_type

P = ParamSpec("P")
T = TypeVar("T")


def log_call(log: logger_type = logging.getLogger("util"), /):
    """log to default logger whenever decorated function is called"""

    def decorator(method: Callable[P, T], /):
        @wraps(method)
        def func(*args: P.args, **kwargs: P.kwargs):
            log_text = f" {method.__name__} "
            log.info(f"{log_text :-^50}")
            return method(*args, **kwargs)

        return func

    return decorator
